package diaswarm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import diaswarm.device.AirPollutionSensor;
import diaswarm.device.CityInfoPanel;

public class Main {

	static List<AirPollutionSensor> pollutionSensors;

	public static void main(String[] args) {
		System.out.println("Starting the AirPollutionSurveillance system ...");
		ComponentBinder componentBinder = new ComponentBinder();

		// Deploying all the application components (i.e., contexts,
		// controllers)
		componentBinder.deployAll();

		// Deploying a city info panel for displaying messages.
		componentBinder.deploy(new CityInfoPanel(componentBinder
				.getServiceConfiguration("CityInfoPanel"), "_cityInfoPanel_1",
				"somewhere"));

		// Deploying air pollution sensors
		pollutionSensors = deployAirPollutionSensors(componentBinder);

		// Each sensor will publish a random aqi value between 0 and 200 every
		// 10 seconds.
		run_scenario();

	}

	private static void run_scenario() {
		while (true)
			publishRandomAqiEveryMillis(10000);
	}

	private static void publishRandomAqiEveryMillis(long millis) {
		try {
			for (AirPollutionSensor airPollutionSensor : pollutionSensors)
				airPollutionSensor.publish(new Random().nextInt(200));
			Thread.sleep(millis); 
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private static List<AirPollutionSensor> deployAirPollutionSensors(
			ComponentBinder componentBinder) {
		List<AirPollutionSensor> pollutionSensorList = new ArrayList<AirPollutionSensor>();

		// Defining 4 default locations in the city : NORTH, SOUTH, EAST, WEST.
		List<String> locations = Arrays
				.asList("NORTH", "SOUTH", "EAST", "WEST");

		// Creating 100 air pollution sensors per location.
		for (String location : locations)
			for (int i = 1; i <= 100; i++) {
				String sensorID = i + "_" + location;
				pollutionSensorList.add(new AirPollutionSensor(componentBinder
						.getServiceConfiguration("AirPollutionSensor_"
								+ sensorID), sensorID, location));
			}

		// Deploying pollution sensors.
		for (AirPollutionSensor airPollutionSensor : pollutionSensorList)
			componentBinder.deploy(airPollutionSensor);

		return pollutionSensorList;
	}

}
