package diaswarm.context;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.mapreduce.MapCollector;
import fr.inria.diagen.core.mapreduce.ReduceCollector;
import fr.inria.phoenix.diasuite.framework.context.averagepollution.AbstractAveragePollution;
import fr.inria.phoenix.diasuite.framework.datatype.locationpollution.LocationPollution;
import fr.inria.phoenix.diasuite.framework.datatype.pollutionlevel.PollutionLevel;

public class AveragePollution extends AbstractAveragePollution {

	public AveragePollution(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);
	}

	// arg0 => key => location
	// arg1 => value => aqi
	// arg2 => output collector => key, Integer
	@Override
	public void map(String arg0, Integer arg1,
			MapCollector<String, Integer> arg2, String... arg3) {
		// Nothing to do at this step, just return the measurement to the reduce
		// phase.
		arg2.emitMap(arg0, arg1);
	}

	// arg0 => key => location
	// arg1 => values
	// arg2 => output collector => key, Float
	@Override
	public void reduce(String arg0, List<Integer> arg1,
			ReduceCollector<String, Float> arg2) {
		int sum = 0;
		int count = arg1.size();

		for (Integer value : arg1)
			sum = sum + value;

		// Emit the average pollution for the given location.
		arg2.emitReduce(arg0, sum / (float) count);
	}

	@Override
	protected List<LocationPollution> onPeriodicAqiFromAirPollutionSensor(
			Map<String, Float> aqiByLocationMap) {
		List<LocationPollution> averagePollutionList = new ArrayList<LocationPollution>();

		// Loop over the map containing average pollution per location
		for (Entry<String, Float> averageAqiPerLocation : aqiByLocationMap
				.entrySet()) {
			LocationPollution locationPollution = null;

			// Choose the pollution level in relation to the air quality index
			if (averageAqiPerLocation.getValue() <= 50)
				locationPollution = new LocationPollution(
						averageAqiPerLocation.getKey(), PollutionLevel.GOOD);
			else if (averageAqiPerLocation.getValue() > 50
					&& averageAqiPerLocation.getValue() <= 100)
				locationPollution = new LocationPollution(
						averageAqiPerLocation.getKey(), PollutionLevel.MODERATE);
			else if (averageAqiPerLocation.getValue() > 100)
				locationPollution = new LocationPollution(
						averageAqiPerLocation.getKey(),
						PollutionLevel.UNHEALTHY);
			
			averagePollutionList.add(locationPollution);
		}

		return averagePollutionList;
	}

}
